@@ -61,6 +61,7 @@ QHash<int,QByteArray> UpdateModel::roleNames() const
     return QAbstractItemModel::roleNames().unite({
         { Qt::CheckStateRole, "checked" },
         { ResourceProgressRole, "resourceProgress" },
+        { ResourceStateRole, "resourceState" },
         { ResourceRole, "resource" },
         { SizeRole, "size" },
         { SectionRole, "section" },
@@ -84,15 +85,16 @@ void UpdateModel::setBackend(ResourcesUpdatesModel* updates)
     activityChanged();
 }
 
-void UpdateModel::resourceHasProgressed(AbstractResource* res, qreal progress)
+void UpdateModel::resourceHasProgressed(AbstractResource* res, qreal progress, AbstractBackendUpdater::State state)
 {
     UpdateItem* item = itemFromResource(res);
     if (!item)
         return;
     item->setProgress(progress);
+    item->setState(state);
 
     const QModelIndex idx = indexFromItem(item);
-    Q_EMIT dataChanged(idx, idx, { ResourceProgressRole, SectionResourceProgressRole });
+    Q_EMIT dataChanged(idx, idx, { ResourceProgressRole, ResourceStateRole, SectionResourceProgressRole });
 }
 
 void UpdateModel::activityChanged()
@@ -135,6 +137,8 @@ QVariant UpdateModel::data(const QModelIndex &index, int role) const
         return QVariant::fromValue<QObject*>(item->resource());
     case ResourceProgressRole:
         return item->progress();
+    case ResourceStateRole:
+        return item->state();
     case ChangelogRole:
         return item->changelog();
     case SectionRole: {
@@ -168,7 +172,7 @@ void UpdateModel::checkResources(const QList<AbstractResource*>& resource, bool
 Qt::ItemFlags UpdateModel::flags(const QModelIndex &index) const
 {
     if (!index.isValid())
-        return nullptr;
+        return Qt::NoItemFlags;
 
     return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 }
@@ -253,9 +257,9 @@ void UpdateModel::setResources(const QList<AbstractResource*>& resources)
         }
     }
     const auto sortUpdateItems = [](UpdateItem *a, UpdateItem *b) { return a->name() < b->name(); };
-    qSort(appItems.begin(), appItems.end(), sortUpdateItems);
-    qSort(systemItems.begin(), systemItems.end(), sortUpdateItems);
-    qSort(addonItems.begin(), addonItems.end(), sortUpdateItems);
+    std::sort(appItems.begin(), appItems.end(), sortUpdateItems);
+    std::sort(systemItems.begin(), systemItems.end(), sortUpdateItems);
+    std::sort(addonItems.begin(), addonItems.end(), sortUpdateItems);
     m_updateItems = (QVector<UpdateItem*>() << appItems << addonItems << systemItems);
     endResetModel();
 
