@@ -57,7 +57,7 @@ public:
     bool addSource(const QString& /*id*/) override { return false; }
     bool removeSource(const QString& /*id*/) override { return false;}
     QString idDescription() override { return QStringLiteral("Snap"); }
-    QList<QAction*> actions() const override { return {}; }
+    QVariantList actions() const override { return {}; }
 
     bool supportsAdding() const override { return false; }
     bool canMoveSources() const override { return false; }
@@ -109,7 +109,7 @@ ResultsStream * SnapBackend::search(const AbstractResourcesBackend::Filters& fil
 
 ResultsStream * SnapBackend::findResourceByPackageName(const QUrl& search)
 {
-    Q_ASSERT(!search.host().isEmpty());
+    Q_ASSERT(!search.host().isEmpty() || !AppStreamUtils::appstreamId(search).isEmpty());
     return search.scheme() == QLatin1String("snap")      ? populate(m_client.find(QSnapdClient::MatchName, search.host())) :
 #ifdef SNAP_FIND_COMMON_ID
            search.scheme() == QLatin1String("appstream") ? populate(m_client.find(QSnapdClient::MatchCommonId, AppStreamUtils::appstreamId(search))) :
